# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack
# TODO: Clean this code

try:
    from PySide6 import QtCore, QtWidgets, QtGui
except ImportError:
    from PySide2 import QtCore, QtWidgets, QtGui

from ftrack_qt.widgets.thumbnails.session_base_thumbnail import (
    SessionThumbnailBase,
)


class EllipseThumbnailBase(SessionThumbnailBase):
    '''Thumbnail which is drawn as an ellipse.'''

    def paintEvent(self, event):
        '''Override paint event to make round thumbnails.'''
        painter = QtGui.QPainter(self)
        painter.setRenderHints(
            QtGui.QPainter.RenderHint.Antialiasing,
            True,
        )

        # TODO: when no image this raises an error because of the assigning of
        #  the empty pixmap. Find another way to solve this.
        brush = QtGui.QBrush(self.pixmap())

        painter.setBrush(brush)

        painter.setPen(QtGui.QPen(QtGui.QColor(0, 0, 0, 0)))

        painter.drawEllipse(QtCore.QRectF(0, 0, self.width(), self.height()))



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack
import os

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore


class EntityInfo(QtWidgets.QWidget):
    '''Widget presenting basic information about an entity(context)'''

    pathReady = QtCore.Signal(object)  # List if context parents are provided

    @property
    def entity(self):
        '''Return the current entity'''
        return self._entity

    @entity.setter
    def entity(self, value):
        '''Set the entity for this widget to *value*'''
        if not value:
            return
        self._entity = value
        parent = value['parent']
        parents = [value]
        while parent is not None:
            parents.append(parent)
            parent = parent['parent']
        parents.reverse()
        self.pathReady.emit(parents)

    def __init__(self, additional_widget=None, parent=None):
        '''
        Instantiate the entity info widget

        :param additional_widget: The optional insertion widget
        :param parent: The parent dialog or frame
        '''
        super(EntityInfo, self).__init__(parent=parent)

        self._entity = None

        self._additional_widget = additional_widget

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.setLayout(QtWidgets.QVBoxLayout())
        self.layout().setContentsMargins(2, 6, 2, 2)
        self.layout().setSpacing(2)

    def build(self):
        name_widget = QtWidgets.QWidget()
        name_widget.setLayout(QtWidgets.QHBoxLayout())
        name_widget.layout().setContentsMargins(0, 0, 0, 0)
        name_widget.layout().setSpacing(0)

        self._name_field = QtWidgets.QLabel()
        self._name_field.setProperty('h3', True)
        name_widget.layout().addWidget(self._name_field)
        if self._additional_widget:
            name_widget.layout().addWidget(self._additional_widget)
        name_widget.layout().addStretch()
        self.layout().addWidget(name_widget)

        self._path_field = QtWidgets.QLabel()
        self._path_field.setProperty('secondary', True)
        self.layout().addWidget(self._path_field)

        self.layout().addStretch()

    def post_build(self):
        self.pathReady.connect(self._on_path_ready)

    def _on_path_ready(self, parents):
        '''Set current path to *names*.'''
        self.set_name_field(parents[-1]['name'])
        self.set_path_field(os.sep.join([p['name'] for p in parents[:-1]]))

    def set_name_field(self, name):
        self._name_field.setText(name)

    def set_path_field(self, path):
        self._path_field.setText(path)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

import platform

from functools import partial

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

import ftrack_constants.qt as qt_constants

from ftrack_qt.utils.widget import center_widget
from ftrack_qt.widgets.dialogs import StyledDialog


class ModalDialog(StyledDialog):
    '''
    A styled modal ftrack dialog box/prompt, intended to live on top of a base dialog or DCC app and
    waits for user input by default
    '''

    background_style = qt_constants.theme.DEFAULT_MODAL_DIALOG_STYLE

    def __init__(self, parent, message=None, question=False, title=None):
        '''
        Initialize a modal dialog either with a message/prompt, or as a base
        for a custom modal dialog, that stays on top of the parent window.

        :param parent: The parent dialog or frame, required.
        :param message: The message or question to show in dialog, if not given it
        is assumed dialog will be further customised (f.e.x. the entity browser)
        :param question: If true, makes dialog behave like a prompt with Yes+No buttons
        :param title: The text to show in dialog title bar
        '''
        self._title_label = None

        super(ModalDialog, self).__init__(parent=parent)

        self.setParent(parent)

        self._message = message
        self._title = title or 'ftrack'
        self._question = question

        self._approve_button = None
        self._deny_button = None

        self.pre_build()
        self.build()
        self.post_build()

        self.setModal(True)
        self.setWindowFlags(
            QtCore.Qt.WindowType.SplashScreen
            | QtCore.Qt.WindowType.WindowStaysOnTopHint
        )

    def pre_build(self):
        self.setLayout(QtWidgets.QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(0)

    def build(self):
        '''Can be overridden by custom dialogs'''
        self._title_label = QtWidgets.QLabel()
        self._title_label.setAlignment(QtCore.Qt.AlignCenter)
        self.layout().addWidget(self._title_label)
        self._title_label.setMinimumHeight(24)

        self.layout().setSpacing(5)

        self.layout().addWidget(self.get_content_widget(), 100)

        buttonbar = QtWidgets.QWidget()
        buttonbar.setLayout(QtWidgets.QHBoxLayout())
        buttonbar.layout().setContentsMargins(10, 1, 10, 1)
        buttonbar.layout().setSpacing(10)

        buttonbar.layout().addWidget(QtWidgets.QLabel(), 100)
        self._approve_button = self.get_approve_button()
        if self._question:
            self._deny_button = self.get_deny_button()
        else:
            self._deny_button = None
        if platform.system().lower() != 'darwin':
            if self._approve_button:
                buttonbar.layout().addWidget(self._approve_button)
            if self._deny_button:
                buttonbar.layout().addWidget(self._deny_button)
        else:
            if self._deny_button:
                buttonbar.layout().addWidget(self._deny_button)
            if self._approve_button:
                buttonbar.layout().addWidget(self._approve_button)

        self.layout().addWidget(buttonbar, 1)

    def get_content_widget(self):
        '''Create dialog main content widget, can be overridden to provide
        custom styled modal dialogs'''
        label = QtWidgets.QLabel(self._message)
        label.setProperty('h3', True)
        return center_widget(label)

    def get_approve_button(self):
        '''Build the approve button widget, can be overridden to provide a
        custom approve button.'''
        button = QtWidgets.QPushButton(
            'YES' if self._question is True else 'OK'
        )
        button.setMinimumSize(QtCore.QSize(40, 35))
        return button

    def get_deny_button(self):
        '''Build the deny (No) button widget, can be overridden to provide a
        custom deny button.'''
        button = QtWidgets.QPushButton('NO')
        button.setMinimumSize(QtCore.QSize(40, 35))
        return button

    def post_build(self):
        if self._approve_button:
            self._approve_button.clicked.connect(partial(self.done, 1))
        if self._deny_button:
            self._deny_button.clicked.connect(self.reject)

        self.setWindowTitle(self._title or '')
        if self._message:
            self.setMaximumHeight(100)
        self.resize(250, 100)

    def setWindowTitle(self, title):
        '''(Override) Set the dialog title'''
        super(ModalDialog, self).setWindowTitle(title)
        if self._title_label:
            self._title_label.setText(title.upper())

    def setVisible(self, visible):
        '''(Override) Set visible, and darken parent if available.'''
        if isinstance(self.parentWidget(), StyledDialog):
            self.parentWidget().darken = visible
        super(ModalDialog, self).setVisible(visible)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

from pathlib import Path
import clique

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

from ftrack_qt.widgets.dialogs import FileDialog
from ftrack_qt.widgets.dialogs import RadioButtonDialog


class FileBrowser(QtWidgets.QWidget):
    '''Browse Widget is a line edit with a browse button'''

    path_changed = QtCore.Signal(object)

    def __init__(self, parent=None):
        '''
        Initialize Browse widget
        '''
        super(FileBrowser, self).__init__(parent=parent)

        self._path_le = None
        self._browse_btn = None

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(15, 0, 0, 0)
        self.layout().setSpacing(0)

    def build(self):
        '''Build widgets'''
        self._path_le = QtWidgets.QLineEdit()
        self._path_le.setAttribute(QtCore.Qt.WA_MacShowFocusRect, False)

        self.layout().addWidget(self._path_le, 20)

        self._browse_btn = QtWidgets.QPushButton('BROWSE')
        self._browse_btn.setProperty('borderless', True)

        self.layout().addWidget(self._browse_btn)

    def post_build(self):
        '''Connect widget signals'''
        self._browse_btn.clicked.connect(self._browse_button_clicked)

    def get_path(self):
        '''Get path from the line edit'''
        return self._path_le.text()

    def set_path(self, path_text):
        '''Set path to the line edit'''
        self._path_le.setText(path_text)

    def set_tool_tip(self, tooltip_text):
        '''Set tooltip'''
        self._path_le.setToolTip(tooltip_text)

    def _browse_button_clicked(self):
        '''Browse button clicked signal'''
        start_dir = self._path_le.text() or Path.home()
        file_dialog = FileDialog(start_dir=str(start_dir), dialog_filter="*")
        selected_path = None
        if len(file_dialog.paths) > 1:
            selected_path = self.process_files_with_clique(file_dialog.paths)
        else:
            selected_path = file_dialog.paths[0]
        self.set_path(selected_path)
        self.path_changed.emit(selected_path)

    def process_files_with_clique(self, file_paths):
        '''Process the selected files with the clique library'''
        collections, remainder = clique.assemble(file_paths)
        collections_list = [collection.format() for collection in collections]
        collections_list.extend(remainder)
        if len(collections_list) > 1:
            radio_button_dialog = RadioButtonDialog(collections_list)
            if radio_button_dialog.exec_():
                selected_item = radio_button_dialog.selected_item()
                return selected_item
            else:
                return None
        else:
            return collections_list[0]



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

# TODO: review this widget code and try to simplify it. Review all the utilities
# as well in the same moment, I think we can simplify it a lot.

from functools import partial

try:
    from PySide6 import QtWidgets, QtCore, QtGui
    import shiboken6 as shiboken
except ImportError:
    from PySide2 import QtWidgets, QtCore, QtGui
    import shiboken2 as shiboken

from ftrack_qt.widgets.thumbnails import ContextThumbnail
from ftrack_qt.widgets.search import SearchBox
from ftrack_qt.widgets.buttons import CircularButton
from ftrack_qt.widgets.overlay import BusyIndicator
from ftrack_qt.widgets.icons import MaterialIcon
from ftrack_qt.widgets.dialogs import ModalDialog
from ftrack_qt.utils.layout import recursive_clear_layout
from ftrack_qt.utils.widget import (
    set_property,
    center_widget,
    InputEventBlockingWidget,
)

from ftrack_utils.threading import BaseThread


class EntityBrowser(ModalDialog):
    '''
    Dialog widget enabling entity/context browsing
    Supports to be driven from an external detached navigator browser (assembler).
    '''

    MODE_ENTITY = 1  # Pick any entity in backend (NIY)
    MODE_CONTEXT = 2  # Pick any context
    MODE_TASK = 3  # Only pick tasks

    SHOW_CHILDREN = False

    entityChanged = QtCore.Signal(
        object
    )  # External; a new context has been set
    rebuildEntityBrowser = QtCore.Signal()  # Entity browser should be rebuilt
    entitiesFetched = QtCore.Signal(
        object
    )  # The entities beneath entity has been fetched

    working = (
        False  # Flag preventing multiple user operations at the same time
    )

    @property
    def entity(self):
        '''Return the current entity'''
        return self._entity

    @entity.setter
    def entity(self, value):
        '''Set the current entity to *value*'''
        if self.working:
            return
        prev_entity = self._entity
        self._entity = value
        self.intermediate_entity = value
        if self._external_navigator is not None:
            self._external_navigator.refreshNavigator.emit()
        if (value or {}).get('id') != (prev_entity or {}).get('id'):
            if prev_entity is not None:
                self.entityChanged.emit(value)
            if self.isVisible():
                self.rebuildEntityBrowser.emit()

    @property
    def entity_id(self):
        '''Return the entity id'''
        return self._entity['id'] if self._entity else None

    @entity_id.setter
    def entity_id(self, value):
        '''Set the current entity id to *value*'''
        self.entity = self.find_context_entity(value)

    @property
    def intermediate_entity(self):
        '''Return the entity currently browsed before user has accepted (applied)'''
        return self._intermediate_entity

    @intermediate_entity.setter
    def intermediate_entity(self, value):
        '''Set the intermediate entity to value'''
        if self.working:
            return
        self._intermediate_entity = value
        self._prev_search_text = ""
        self._search.text = ""
        self._selected_entity = value
        if self.isVisible():
            # Rebuild navigator and browsing content based on current intermediate entity
            self._navigator.refreshNavigator.emit()
            self.rebuildEntityBrowser.emit()

    @property
    def mode(self):
        '''Return the entity browser mode'''
        return self._mode

    @mode.setter
    def mode(self, value):
        '''Set the entity browser mode to value'''
        if value not in [
            EntityBrowser.MODE_ENTITY,
            EntityBrowser.MODE_CONTEXT,
            EntityBrowser.MODE_TASK,
        ]:
            raise Exception("Unknown entity browser mode - !{}".format(value))
        self._mode = value

    @property
    def session(self):
        '''Return :class:`ftrack_api.session.Session`'''
        return self._session

    def __init__(self, parent, session, entity=None, mode=None, title=None):
        '''
        Initialize the entity browser

        :param parent: The parent dialog or frame
        :param session: :class:`ftrack_api.session.Session`
        :param entity: The entity to use from start
        :param mode: Set the entity browser mode
        :param title: The text to put on title bar
        '''
        self._entity = None  # The entity that has been set and applied
        self._intermediate_entity = (
            None  # The intermediate entity currently browsed
        )
        self._selected_entity = None  # The selected entity
        self._session = session
        self._external_navigator = None
        self._prev_search_text = ""
        self.working = False

        self.mode = mode or EntityBrowser.MODE_TASK

        super(EntityBrowser, self).__init__(
            parent, question=True, title=title or 'ftrack Entity Browser'
        )

        if entity:
            self.entity = entity

        self.setAttribute(QtCore.Qt.WA_DeleteOnClose, False)

    def pre_build(self):
        super(EntityBrowser, self).pre_build()
        # The navigator within dialog, showing intermediate entity
        self._navigator = EntityBrowserNavigator(self, is_browser=True)

    def get_content_widget(self):
        '''(Override)'''
        self._content_widget = InputEventBlockingWidget(lambda: self.working)
        self._content_widget.setLayout(QtWidgets.QVBoxLayout())
        self._content_widget.layout().setContentsMargins(1, 1, 1, 1)
        self._content_widget.layout().setSpacing(5)

        toolbar = QtWidgets.QWidget()
        toolbar.setLayout(QtWidgets.QHBoxLayout())
        toolbar.layout().setContentsMargins(0, 0, 0, 0)
        toolbar.layout().setSpacing(1)

        toolbar.layout().addWidget(self._navigator)
        toolbar.layout().addWidget(QtWidgets.QLabel(), 100)
        self._rebuild_button = CircularButton('sync')
        toolbar.layout().addWidget(self._rebuild_button)

        self._content_widget.layout().addWidget(toolbar)

        self._search = SearchBox(collapsed=False, collapsable=False)
        self._search.inputUpdated.connect(self._on_search)
        self._content_widget.layout().addWidget(self._search)

        self._scroll = QtWidgets.QScrollArea()
        self._scroll.setStyle(QtWidgets.QStyleFactory.create("plastique"))
        self._scroll.setWidgetResizable(True)
        self._scroll.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)

        self._content_widget.layout().addWidget(self._scroll, 100)

        return self._content_widget

    def get_approve_button(self):
        '''(Override)'''
        approve_button = QtWidgets.QPushButton("APPLY CONTEXT")
        approve_button.setMinimumSize(QtCore.QSize(80, 35))
        return approve_button

    def get_deny_button(self):
        '''(Override)'''
        deny_button = QtWidgets.QPushButton("CANCEL")
        deny_button.setMinimumSize(QtCore.QSize(80, 35))
        return deny_button

    def post_build(self):
        super(EntityBrowser, self).post_build()

        self.rebuildEntityBrowser.connect(self.rebuild)

        self._navigator.changeEntity.connect(self._on_set_intermediate_entity)
        self._navigator.removeEntity.connect(
            self._set_parent_intermediate_entity
        )

        self._rebuild_button.clicked.connect(self.rebuild)
        self._approve_button.clicked.connect(self._on_apply)
        self._deny_button.clicked.connect(self.reject)

        self.entitiesFetched.connect(self._on_entities_fetched)

        self.resize(700, 450)

    def create_navigator(self):
        '''Create and return an external navigator widget (Assembler browser)'''
        self._external_navigator = EntityBrowserNavigator(self)
        self._external_navigator.changeEntity.connect(self._browse_parent)
        self._external_navigator.addEntity.connect(self._add_entity)
        self._external_navigator.removeEntity.connect(self._set_parent_entity)
        self._external_navigator.refreshNavigator.emit()
        return self._external_navigator

    def _browse_parent(self, entity):
        '''Browse to the parent of *entity*'''
        if self.working:
            return
        self.intermediate_entity = entity['parent'] if entity else None
        self.show()

    def _add_entity(self):
        '''Add entity were clicked in external navigator, show entity browser'''
        if self.working:
            return
        self.show()

    def _set_parent_entity(self, entity):
        '''Set the current entity to the parent of *entity*'''
        if self.working:
            return
        self.entity = entity['parent'] if entity else None

    def show(self):
        '''Show dialog, on the entity supplied'''
        self._navigator.refreshNavigator.emit()
        self.rebuildEntityBrowser.emit()
        return super(EntityBrowser, self).show()

    def exec_(self):
        '''(Override)'''
        try:
            self._navigator.refreshNavigator.emit()
            self.rebuildEntityBrowser.emit()
            return super(EntityBrowser, self).exec_()
        finally:
            # Make sure event filters are removed
            self._content_widget.stop()
            self._navigator.stop()

    def _set_parent_intermediate_entity(self, entity):
        '''Set the current intermediate entity to the parent of *entity*'''
        self.intermediate_entity = entity['parent'] if entity else None

    def find_context_entity(self, entity_id):
        '''Fetch the entity from frack based on *entity_id*'''
        context_entity = self.session.query(
            'select link, name, parent, parent.name from Context where id '
            'is "{}"'.format(entity_id)
        ).one()
        return context_entity

    def rebuild(self):
        if self.working:
            return

        self.working = True
        self.entity_widgets = []

        self._busy_indicator = BusyIndicator(False)

        self._scroll.setWidget(center_widget(self._busy_indicator, 30, 30))

        self._busy_indicator.start()

        self.update()

        # Resolve version this context is depending on in separate thread
        thread = BaseThread(
            name='fetch_entities_thread',
            target=self._fetch_entities,
            target_args=(),
        )
        thread.start()

    def _fetch_entities(self):
        '''(Run in background thread) Fetch projects/child entities'''
        signal_emitted = False
        try:
            intermediate_entity = self.intermediate_entity
            if self.intermediate_entity is None:
                # List projects
                entities = self.session.query(
                    'select id, name from Project where status=active'
                ).all()
            else:
                entities = self.session.query(
                    'select id, name{} from Context where parent.id is {}'.format(
                        ', children' if self.SHOW_CHILDREN else '',
                        intermediate_entity['id'],
                    )
                ).all()
            self.entitiesFetched.emit(entities)
            signal_emitted = True
        finally:
            if not signal_emitted:
                # Crashed, stop working to unblock
                self.working = False

    def _on_entities_fetched(self, entities):
        '''Entities has been fetch, rebuild widget'''
        try:
            entities_widget = QtWidgets.QWidget()
            entities_widget.setLayout(QtWidgets.QVBoxLayout())
            entities_widget.layout().setContentsMargins(0, 0, 0, 0)
            entities_widget.layout().setSpacing(0)

            self._busy_indicator.stop()

            self._scroll.setWidget(entities_widget)

            self.entity_widgets = []
            if (
                self.intermediate_entity is not None
                and self.intermediate_entity.get('parent') is not None
            ):
                parent_entity_widget = EntityWidget(
                    self.intermediate_entity['parent'],
                    False,
                    self,
                    is_parent=True,
                )
                parent_entity_widget.clicked.connect(
                    partial(
                        self._entity_selected,
                        self.intermediate_entity['parent'],
                    )
                )
                parent_entity_widget.doubleClicked.connect(
                    partial(
                        self._entity_selected,
                        self.intermediate_entity['parent'],
                        True,
                    )
                )
                entities_widget.layout().addWidget(parent_entity_widget)
                self.entity_widgets.append(parent_entity_widget)
            for entity in entities:
                entity_widget = EntityWidget(entity, False, self)
                entity_widget.clicked.connect(
                    partial(self._entity_selected, entity)
                )
                entity_widget.doubleClicked.connect(
                    partial(self._entity_selected, entity, True)
                )
                entities_widget.layout().addWidget(entity_widget)
                self.entity_widgets.append(entity_widget)
                if self.SHOW_CHILDREN:
                    for sub_entity in entity['children']:
                        if sub_entity.entity_type == 'Task':
                            sub_entity_widget = EntityWidget(
                                sub_entity, True, self
                            )
                            sub_entity_widget.clicked.connect(
                                partial(self._entity_selected, sub_entity)
                            )
                            entities_widget.layout().addWidget(
                                sub_entity_widget
                            )
                            self.entity_widgets.append(sub_entity_widget)

            entities_widget.layout().addWidget(QtWidgets.QLabel(), 100)

        finally:
            self.working = False

    def refresh(self):
        '''Filter visible entities on search.'''
        text = self._search.text.lower()
        for entity_widget in self.entity_widgets:
            entity_widget.setVisible(
                len(text) == 0
                or entity_widget.entity['name'].lower().find(text) > -1
            )

    def _entity_selected(self, entity, double_click=False):
        '''User has selected an entity'''
        self._selected_entity = entity
        for entity_widget in self.entity_widgets:
            set_property(
                entity_widget,
                "selected",
                True if entity_widget.entity['id'] == entity['id'] else False,
            )
        if entity.entity_type != "Task":
            # Dive further down
            thread = BaseThread(
                name='entity_clicked_thread',
                target=self._on_set_intermediate_entity,
                target_args=[entity],
            )
            thread.start()
        else:
            self.update()
            if double_click:
                if self.mode == EntityBrowser.MODE_TASK:
                    self._on_apply()

    def _on_set_intermediate_entity(self, entity):
        '''Callback for setting the intermediate entity'''
        self.intermediate_entity = entity

    def update(self):
        '''Update entity browser'''
        # Search text changed?
        if self._search.text != self._prev_search_text:
            self.refresh()
            self._prev_search_text = self._search.text
        has_selection = False
        if self._selected_entity:
            if self.mode == EntityBrowser.MODE_CONTEXT:
                has_selection = True
            elif self.mode == EntityBrowser.MODE_TASK:
                has_selection = self._selected_entity.entity_type == 'Task'
        self._approve_button.setEnabled(has_selection)

    def _on_search(self, text):
        '''Filter list of entities - only include the ones matching text.'''
        self.update()

    def _on_apply(self):
        '''Set the immediate entity and close dialog'''
        if self.working:
            return
        self.entity = self._selected_entity
        self.done(1)


class EntityBrowserNavigator(InputEventBlockingWidget):
    '''Widget representing the path down to the current entity (intermediate if not external browser)'''

    changeEntity = QtCore.Signal(
        object
    )  # A entity navigator item has been clicked
    removeEntity = QtCore.Signal(
        object
    )  # The remove button has been clicked on an entity navigator item
    addEntity = (
        QtCore.Signal()
    )  # The plus sign button has been clicked (external navigator only)
    refreshNavigator = QtCore.Signal()  # Navigator should be refreshed
    rebuildNavigator = QtCore.Signal()  # Navigator should be rebuilt

    working = False

    @property
    def entity(self):
        '''Retrieve the entity browsed in navigator'''
        return (
            self._entity_browser.intermediate_entity
            if self._is_browser
            else self._entity_browser.entity
        )

    @property
    def entity_browser(self):
        '''Return parent entity browser driving this navigator'''
        return self._entity_browser

    @property
    def session(self):
        '''Return the :class:`ftrack_api.session.Session`'''
        return self._entity_browser.session

    def __init__(
        self,
        entity_browser,
        is_browser=False,
        parent=None,
    ):
        '''
        Initialise EntityBrowser navigator widget

        :param entity_browser: :class:`~ftrack_framework_qt.ui.utility.widget.entity_browser.EntityBrowser` instance
        :param is_browser: If False (default), navigator is assumed to be an external navigator docked in another widget. If True, navigator resides withing the entity browser header.
        :param parent: The parent dialog or frame
        '''
        super(EntityBrowserNavigator, self).__init__(
            lambda: self.working or self._entity_browser.working, parent=parent
        )
        self._entity_browser = entity_browser
        self._is_browser = is_browser

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.setLayout(self.create_layout())

    def create_layout(self):
        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)
        return layout

    def build(self):
        pass

    def post_build(self):
        self.setMinimumHeight(36)
        self.setMaximumHeight(36)
        self.refreshNavigator.connect(self.refresh)
        self.rebuildNavigator.connect(self.rebuild)

    def refresh(self):
        '''Rebuild the navigator'''

        self.working = True

        # self.setLayout(self.create_layout())
        recursive_clear_layout(self.layout())

        thread = BaseThread(
            name='fetch_entities_thread',
            target=self._on_entities_fetched,
            target_args=(),
        )
        thread.start()

    def _on_entities_fetched(self):
        self.rebuildNavigator.emit()

    def rebuild(self):
        '''Rebuild the navigator'''
        try:
            # Rebuild widget

            if self._is_browser:
                home_button = HomeContextButton()
                home_button.clicked.connect(self._on_go_home)
                self.layout().addWidget(home_button)

                l_arrow = QtWidgets.QLabel()
                l_arrow.setPixmap(
                    MaterialIcon('chevron-right', color='#676B70').pixmap(
                        QtCore.QSize(16, 16)
                    )
                )
                l_arrow.setMinimumSize(QtCore.QSize(16, 16))
                self.layout().addWidget(l_arrow)

            add_enabled = not self._is_browser

            if self.entity:
                for index, link in enumerate(self.entity['link']):
                    button = NavigationEntityButton(link)
                    button.clicked.connect(
                        partial(self._on_entity_changed, button.link_entity)
                    )
                    set_property(
                        button, 'first', 'true' if index == 0 else 'false'
                    )
                    if link['type'] != 'Project':
                        button.remove_button.released.connect(
                            partial(self._on_remove_entity, link)
                        )
                    self.layout().addWidget(button)

                    if index < len(self.entity['link']) - 1:
                        l_arrow = QtWidgets.QLabel()
                        l_arrow.setPixmap(
                            MaterialIcon(
                                'chevron-right', color='#676B70'
                            ).pixmap(QtCore.QSize(16, 16))
                        )
                        l_arrow.setMinimumSize(QtCore.QSize(16, 16))
                        self.layout().addWidget(l_arrow)

            if add_enabled:
                if self.entity:
                    l_arrow = QtWidgets.QLabel()
                    l_arrow.setPixmap(
                        MaterialIcon('chevron-right', color='#676B70').pixmap(
                            QtCore.QSize(16, 16)
                        )
                    )
                    l_arrow.setMinimumSize(QtCore.QSize(16, 16))
                    self.layout().addWidget(l_arrow)

                add_button = AddContextButton()
                add_button.clicked.connect(self._on_add_entity)
                self.layout().addWidget(add_button)

            self.layout().addWidget(QtWidgets.QLabel(), 100)
        finally:
            self.working = False

    def _on_go_home(self):
        if self.working or self._entity_browser.working:
            return
        self.changeEntity.emit(None)

    def _on_add_entity(self, **kwargs):
        '''The add entity button were clicked in navigator'''
        if self.working or self._entity_browser.working:
            return
        self.addEntity.emit()

    def _on_entity_changed(self, link_entity):
        '''A navigator item has been clicked.'''
        if self.working or self._entity_browser.working:
            return
        self.changeEntity.emit(
            self._entity_browser.find_context_entity(link_entity['id'])
        )

    def _on_remove_entity(self, link_entity):
        if self.working or self._entity_browser.working:
            return
        self.removeEntity.emit(
            self._entity_browser.find_context_entity(link_entity['id'])
        )


class NavigationEntityButton(QtWidgets.QFrame):
    '''A button representing a context within the navigator.'''

    clicked = QtCore.Signal()

    def __init__(self, link_entity, parent=None):
        super(NavigationEntityButton, self).__init__(parent=parent)
        self.link_entity = link_entity
        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(3, 0, 3, 0)
        self.layout().setSpacing(4)

    def build(self):
        label = QtWidgets.QLabel(self.link_entity['name'])
        self.layout().addWidget(label)
        if self.link_entity['type'] != 'Project':
            self.layout().addStretch()
            self.remove_button = QtWidgets.QPushButton(
                MaterialIcon('close', color='#94979a'), ""
            )
            self.remove_button.setFixedSize(8, 8)
            self.layout().addWidget(self.remove_button)

    def post_build(self):
        fm = QtGui.QFontMetrics(self.font())
        self.setMinimumWidth(
            fm.horizontalAdvance(self.link_entity['name'])
            + (4 if self.link_entity['type'] != 'Project' else 0)
        )
        self.setMinimumHeight(25)
        self.setMaximumHeight(25)

    def mousePressEvent(self, event):
        self.clicked.emit()
        return super(NavigationEntityButton, self).mousePressEvent(event)


class EntityWidget(QtWidgets.QFrame):
    '''A button representing a context within the navigator.'''

    clicked = QtCore.Signal()
    doubleClicked = QtCore.Signal()

    def __init__(
        self, entity, is_sub_task, entity_browser, parent=None, is_parent=False
    ):
        super(EntityWidget, self).__init__(parent=parent)
        self.setProperty('selectable', True)
        self.setProperty('border', True)
        self.entity = entity
        self.is_parent = is_parent
        self.is_sub_task = is_sub_task
        self._entity_browser = entity_browser
        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(3, 1, 1, 1)
        self.layout().setSpacing(2)

    def build(self):
        self.thumbnail_widget = ContextThumbnail(self.entity.session)
        self.thumbnail_widget.setMinimumWidth(71)
        self.thumbnail_widget.setMinimumHeight(40)
        self.thumbnail_widget.setMaximumWidth(71)
        self.thumbnail_widget.setMaximumHeight(40)
        if not self.is_parent:
            self.thumbnail_widget.load(self.entity['id'])
        self.layout().addWidget(self.thumbnail_widget)

        central_widget = QtWidgets.QWidget()
        central_widget.setLayout(QtWidgets.QVBoxLayout())
        central_widget.layout().setContentsMargins(3, 0, 0, 0)
        central_widget.layout().setSpacing(0)

        # Context name
        label = QtWidgets.QLabel(
            self.entity['name'] if not self.is_parent else '..'
        )
        label.setProperty('h3' if not self.is_parent else 'h2', True)
        central_widget.layout().addWidget(label)

        lower_widget = QtWidgets.QWidget()
        lower_widget.setLayout(QtWidgets.QHBoxLayout())
        lower_widget.layout().setContentsMargins(3, 0, 0, 0)
        lower_widget.layout().setSpacing(0)

        if self.is_sub_task and not self.is_parent:
            # Sub path
            sub_path = QtWidgets.QLabel(
                '.. / {}'.format(self.entity['parent']['name'])
            )
            sub_path.setProperty('secondary', True)
            lower_widget.layout().addWidget(sub_path)

        if not self.is_parent:
            type_widget = TypeWidget(self.entity)
            lower_widget.layout().addWidget(type_widget)

        lower_widget.layout().addWidget(QtWidgets.QLabel(), 100)

        central_widget.layout().addWidget(lower_widget)

        self.layout().addWidget(central_widget, 100)

        if self.entity.entity_type != 'Task' and not self.is_parent:
            l_arrow = QtWidgets.QLabel()
            l_arrow.setPixmap(
                MaterialIcon('chevron-right', color='#676B70').pixmap(
                    QtCore.QSize(16, 16)
                )
            )
            l_arrow.setMinimumSize(QtCore.QSize(16, 16))
            self.layout().addWidget(l_arrow)

    def post_build(self):
        if self.entity.entity_type == "Task":
            # Show task status
            self.setStyleSheet(
                '''
                EntityWidget {
                    border-right: 3px solid %s;
                }'''
                % (self.entity['status']['color'])
            )
        self.setMinimumHeight(45 if not self.is_parent else 20)
        self.setMaximumHeight(45 if not self.is_parent else 20)

    def mousePressEvent(self, event):
        if not shiboken.isValid(self) or not shiboken.isValid(
            super(EntityWidget, self)
        ):
            # Widget has been destroyed
            return
        retval = super(EntityWidget, self).mousePressEvent(event)
        if (
            self._entity_browser is None
            or self._entity_browser.working is False
        ):
            self.clicked.emit()
        return retval

    def mouseDoubleClickEvent(self, event):
        if not shiboken.isValid(self) or not shiboken.isValid(
            super(EntityWidget, self)
        ):
            # Widget has been destroyed
            return
        return super(EntityWidget, self).mouseDoubleClickEvent(event)
        self.doubleClicked.emit()


class AddContextButton(CircularButton):
    def __init__(self, parent=None):
        super(AddContextButton, self).__init__('add', '#eee', parent=parent)

    def get_border_style(self, color):
        return '''            
            border: 1 dashed #2b3036;
            border-radius: 16px;
        '''


class HomeContextButton(CircularButton):
    def __init__(self, parent=None):
        super(HomeContextButton, self).__init__('home', '#eee', parent=parent)

    def get_border_style(self, color):
        return '''            
            border: 1 dashed #2b3036;
            border-radius: 16px;
        '''


class TypeWidget(QtWidgets.QFrame):
    '''Widget representing the entity type with text and icon'''

    def __init__(self, entity, parent=None):
        super(TypeWidget, self).__init__(parent=parent)
        self._entity = entity
        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(3, 0, 7, 0)
        self.layout().setSpacing(2)

    def build(self):
        l_icon = QtWidgets.QLabel()

        if self._entity.entity_type != 'Task':
            icon_name = "help_outline"
            if self._entity.entity_type == 'Project':
                icon_name = "home"
            elif self._entity.entity_type == 'Folder':
                icon_name = "folder"
            elif self._entity.entity_type == 'Episode':
                icon_name = "smart_display"
            elif self._entity.entity_type == 'Sequence':
                icon_name = "movie"
            elif self._entity.entity_type == 'Shot':
                icon_name = "movie"
            elif self._entity.entity_type == "AssetBuild":
                icon_name = "table_chart"
            l_icon.setPixmap(
                MaterialIcon(icon_name, color='#FFDD86').pixmap(
                    QtCore.QSize(16, 16)
                )
            )
        else:
            l_icon.setPixmap(
                MaterialIcon(
                    'assignment_turned_in',
                    variant='outlined',
                    color=self._entity['type']['color'],
                ).pixmap(QtCore.QSize(14, 14))
            )
        self.layout().addWidget(l_icon)
        l_name = QtWidgets.QLabel(
            self._entity.entity_type
            if self._entity.entity_type != 'Task'
            else self._entity['type']['name']
        )
        self.layout().addWidget(l_name)

    def post_build(self):
        self.setMinimumHeight(25)
        self.setMaximumHeight(25)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack


def recursive_clear_layout(layout, removed_widgets=[]):
    '''Recursively remove all widgets from the *layout*'''
    for i in reversed(range(layout.count())):
        item = layout.takeAt(i)
        widget = item.widget()
        if widget is not None:
            if hasattr(widget, 'widget'):
                if (
                    hasattr(widget.widget(), 'layout')
                    and widget.widget().layout()
                ):
                    recursive_clear_layout(
                        widget.widget().layout(), removed_widgets
                    )
            elif hasattr(widget, 'layout') and widget.layout():
                recursive_clear_layout(widget.layout(), removed_widgets)
            removed_widgets.append(widget)
            widget.deleteLater()
        else:
            if hasattr(item, 'layout') and item.layout():
                recursive_clear_layout(item.layout(), removed_widgets)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

from ftrack_qt.widgets.icons import MaterialIcon


class CircularButton(QtWidgets.QPushButton):
    '''Widget representing a circular button with an outline'''

    def __init__(
        self, icon_name, color=None, diameter=32, variant=None, parent=None
    ):
        '''
        Initialize circular button

        :param icon_name: The material icon name
        :param color: The color, in html #RRGGBB format, or rgba(r,g,b,alpha)
        :param diameter: The circle diameter in pixels
        :param variant: The material icon variant to use (optional)
        :param parent: The parent dialog or frame
        '''
        super(CircularButton, self).__init__(parent)

        self.color = color

        self.setMaximumSize(QtCore.QSize(diameter, diameter))
        self.setMinimumSize(QtCore.QSize(diameter, diameter))

        if color is None:
            color = '#D3D3D3'
            border_color = 'rgba(255, 255, 255, 0.1)'
        else:
            border_color = color

        self.setIcon(MaterialIcon(icon_name, variant=variant, color=color))

        self.setStyleSheet(
            '''
            color: {};
            {}
            '''.format(
                color, self.get_border_style(border_color)
            )
        )

    def get_border_style(self, color):
        return '''
            border: 1px solid {};
            border-radius: 16px;
        '''.format(
            color
        )



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore, QtGui
except ImportError:
    from PySide2 import QtWidgets, QtCore, QtGui


from ftrack_qt.widgets.icons import MaterialIcon


class OverlayWidget(QtWidgets.QFrame):
    '''
    Empty widget holder ready to parent a widget with a close button
    '''

    close_button_clicked = QtCore.Signal()

    def __init__(self, parent=None):
        '''
        Initialize Overlay
        '''
        super(OverlayWidget, self).__init__(parent=parent)

        self._container_widget = None
        self._close_btn = None
        self._fill_color = None
        self._pen_color = None
        self._event_filter_installed = False

        self.build()
        self.post_build()

    def build(self):
        main_layout = QtWidgets.QVBoxLayout()

        h_layout = QtWidgets.QHBoxLayout()
        h_layout.setContentsMargins(0, 0, 0, 0)
        h_layout.addStretch()

        self._close_btn = QtWidgets.QPushButton('')
        self._close_btn.setIcon(MaterialIcon('close', color='#FFDD86'))
        self._close_btn.setProperty('borderless', True)
        self._close_btn.setFixedSize(24, 24)

        h_layout.addWidget(self._close_btn)

        main_layout.addLayout(h_layout)
        self.setLayout(main_layout)

    def post_build(self):
        self._close_btn.clicked.connect(self.on_close_button_callback)

    def on_close_button_callback(self):
        '''Emit a signal when close button is clicked'''
        self.close_button_clicked.emit()

    def set_widget(self, widget):
        '''Set the given *widget* into the main layout'''
        self.layout().addWidget(widget)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack
# TODO: Clean this code

import urllib.request, urllib.parse, urllib.error

try:
    from PySide6 import QtCore, QtWidgets
except ImportError:
    from PySide2 import QtCore, QtWidgets

from ftrack_qt.widgets.thumbnails.session_base_thumbnail import (
    SessionThumbnailBase,
)


class ContextThumbnail(SessionThumbnailBase):
    '''Context thumbnail widget'''

    def _download(self, reference):
        '''Return thumbnail from *reference*.'''
        context = self.session.get('Context', reference)
        thumbnail = context['thumbnail']
        if thumbnail is None and context['thumbnail_id'] is not None:
            thumbnail = self.session.query(
                'FileComponent where id is "{}"'.format(
                    context['thumbnail_id']
                )
            ).one()
        url = self.get_thumbnail_url(thumbnail)
        if url is not None:
            return super(ContextThumbnail, self)._download(url)
        else:
            raise urllib.error.URLError("No context URL")

    def get_thumbnail_url(self, component):
        if not component:
            return

        params = urllib.parse.urlencode(
            {
                'id': component['id'],
                'username': self.session.api_user,
                'apiKey': self.session.api_key,
            }
        )

        result_url = '{base_url}/component/thumbnail?{params}'.format(
            base_url=self.session._server_url, params=params
        )
        return result_url

    def _scaleAndSetPixmap(self, pixmap):
        '''Scale and set *pixmap*.'''
        if self._scale:
            scaled_pixmap = pixmap.scaled(
                self.size(),
                QtCore.Qt.AspectRatioMode.KeepAspectRatio,
                QtCore.Qt.TransformationMode.SmoothTransformation,
            )
        else:
            scaled_pixmap = pixmap
        self.setPixmap(scaled_pixmap)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore


class RadioButtonDialog(QtWidgets.QDialog):
    '''Dialog to select item from multiple items found.'''

    @property
    def items(self):
        '''Return items of the dialog'''
        return self._items

    def __init__(self, items, parent=None):
        '''
        Initialize dialog
        '''
        super(RadioButtonDialog, self).__init__(parent=parent)

        self.setWindowTitle('Select Item')

        self._items = items

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.layout = QtWidgets.QVBoxLayout(self)
        self.setLayout(self.layout)

    def build(self):
        '''Build widgets'''

        message = QtWidgets.QLabel('Multiple items found, please select one: ')
        self.layout.addWidget(message)

        self.radio_buttons = []
        self.button_group = QtWidgets.QButtonGroup(self)

        for item in self.items:
            radio_button = QtWidgets.QRadioButton(item)
            self.radio_buttons.append(radio_button)
            self.button_group.addButton(radio_button)
            self.layout.addWidget(radio_button)

        self.ok_button = QtWidgets.QPushButton('OK')

        self.button_layout = QtWidgets.QHBoxLayout()
        self.button_layout.addStretch()
        self.button_layout.addWidget(self.ok_button)

        self.layout.addLayout(self.button_layout)

    def post_build(self):
        self.ok_button.clicked.connect(self.accept)

    def selected_item(self):
        for radio_button in self.radio_buttons:
            if radio_button.isChecked():
                return radio_button.text()
        return None



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

from ftrack_qt.utils.widget import set_property


class NewAssetInput(QtWidgets.QFrame):
    '''Widget holding new asset input during publish'''

    active_changed = QtCore.Signal(object)
    '''Signal emitted when the input is activated or deactivated, with active as argument.'''

    text_changed = QtCore.Signal(object)
    '''Signal emitted when the text is changed, with text as argument.'''

    @property
    def active(self):
        '''Return whether the input is active.'''
        return self._active

    @active.setter
    def active(self, value):
        '''Set the input as active or inactive.'''
        if self._active == value:
            return
        self._active = value
        self._button.setEnabled(not value)
        self._name.setVisible(value)
        self._version_label.setVisible(value)
        self._filler.setVisible(not value)
        self._button.setEnabled(not value)
        if value:
            self._name.setFocus()
            self.text_changed.emit(self._name.text())
        self.active_changed.emit(value)

    def __init__(self, validator, placeholder_name):
        '''Initialize the NewAssetInput widget.'''
        super(NewAssetInput, self).__init__()

        self._validator = validator
        self._placeholder_name = placeholder_name
        self._active = False

        self._button = None
        self._name = None
        self._version_label = None

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        '''Set up layout and dimensions before building.'''
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(4, 1, 1, 1)
        self.layout().setSpacing(1)
        self.setMaximumHeight(32)

    def build(self):
        '''Build the button, name input, and version label.'''
        self._button = QtWidgets.QPushButton('NEW')
        self._button.setProperty('filled', True)
        self._button.setFixedSize(56, 30)
        self._button.setMaximumSize(56, 30)

        self.layout().addWidget(self._button)

        self._name = QtWidgets.QLineEdit()
        self._name.setAttribute(QtCore.Qt.WA_MacShowFocusRect, False)
        self._name.setPlaceholderText(self._placeholder_name)
        self._name.setValidator(self._validator)
        self._name.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Preferred,
            QtWidgets.QSizePolicy.Policy.Minimum,
        )
        self._name.setVisible(False)
        self.layout().addWidget(self._name, 1000)

        self._version_label = QtWidgets.QLabel('- Version 1')
        self._version_label.setVisible(False)
        self.layout().addWidget(self._version_label)

        self._filler = QtWidgets.QLabel('')
        self._filler.setSizePolicy(
            QtWidgets.QSizePolicy.Policy.Expanding,
            QtWidgets.QSizePolicy.Policy.Minimum,
        )
        self.layout().addWidget(self._filler, 100)

    def post_build(self):
        '''Connect signals and callbacks after building.'''
        self._button.clicked.connect(self.input_clicked)
        self._name.mousePressEvent = self.input_clicked
        self._name.textChanged.connect(self.on_text_changed)

    def mousePressEvent(self, event):
        '''Override mouse press to emit signal.'''
        super(NewAssetInput, self).mousePressEvent(event)
        self.active = True
        self.text_changed.emit(self._name.text())

    def input_clicked(self, event):
        '''Callback on user button or name click.'''
        self.active = True
        self.text_changed.emit(self._name.text())

    def on_text_changed(self):
        '''Emit signal when text is changed.'''
        self.text_changed.emit(self._name.text())

    def set_valid(self, valid):
        '''Set the input field as valid or invalid.'''
        if valid:
            set_property(self._name, 'input', '')
        else:
            set_property(self._name, 'input', 'invalid')

    def set_default_name(self, name):
        '''Set the default name for the input field.'''
        self._name.setText(name)



from ftrack_qt.widgets.thumbnails.asset_version_thumbnail import (
    AssetVersionThumbnail,
)
from ftrack_qt.widgets.thumbnails.base_thumbnail import ThumbnailBase
from ftrack_qt.widgets.thumbnails.context_thumbnail import ContextThumbnail
from ftrack_qt.widgets.thumbnails.ellipse_thumbnail import EllipseThumbnailBase
from ftrack_qt.widgets.thumbnails.user_thumbnail import UserThumbnail
from ftrack_qt.widgets.thumbnails.session_base_thumbnail import (
    SessionThumbnailBase,
)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

import logging

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore


class AssetList(QtWidgets.QListWidget):
    '''Widget presenting list of existing assets'''

    active_changed = QtCore.Signal(object)
    '''Signal emitted when the list is activated or deactivated, with active as argument.'''

    version_changed = QtCore.Signal(object)
    '''Signal emitted when version is changed, with assetversion entity as 
    argument (version select mode)'''

    assets_added = QtCore.Signal(object)
    '''Signal emitted when assets are added to the list'''

    selected_item_changed = QtCore.Signal(object, object, object)
    '''Signal emitted when selected item is changed with arguments containing 
    the index of the current version, the version dictionary and the asset_id'''

    @property
    def latest_published_asset_item(self):
        return self._latest_published_asset_item

    @property
    def active(self):
        '''Return if list is active - focus or not'''
        return self._active

    @active.setter
    def active(self, value):
        '''Set active state of the list'''
        if value == self._active:
            return
        self._active = value
        # Activate or deactivate the list items
        for idx in range(self.count()):
            item = self.item(idx)
            widget = self.itemWidget(item)
            if hasattr(widget, 'active'):
                widget.active = value
        if value:
            # Must have something selected
            if self.currentRow() == -1:
                self.setCurrentItem(self.item(0))
        else:
            self.blockSignals(True)
            self.setCurrentRow(-1)  # Make sure list is deselected
            self.blockSignals(False)
        self.active_changed.emit(value)

    def __init__(self, asset_list_widget_item_class, parent=None):
        '''Initialize AssetList'''
        super(AssetList, self).__init__(parent=parent)
        self.logger = logging.getLogger(
            __name__ + '.' + self.__class__.__name__
        )
        self.asset_list_widget_item_class = asset_list_widget_item_class
        self._latest_published_asset_item = None
        self._active = True

        self.setHorizontalScrollBarPolicy(
            QtCore.Qt.ScrollBarPolicy.ScrollBarAlwaysOff
        )
        # TODO: Investigate further on how to make it look correctly
        # self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        # self.verticalScrollBar().setDisabled(True)
        self.setAutoScroll(True)

        self.setSpacing(1)
        self.assets = []
        self.currentItemChanged.connect(self.on_item_changed_callback)

    def set_assets(self, assets):
        '''Set assets for the list'''
        self.currentItemChanged.disconnect()
        self._latest_published_asset_item = None
        self.clear()
        self.assets = assets

        for asset_id, asset_dict in self.assets.items():
            widget = self.asset_list_widget_item_class(
                asset_name=asset_dict['name'],
                asset_id=asset_id,
                versions=asset_dict['versions'],
                server_url=asset_dict['server_url'],
            )

            if hasattr(widget, 'version_changed'):
                widget.version_changed.connect(
                    self._on_version_changed_callback
                )
            list_item = QtWidgets.QListWidgetItem(self)
            list_item.setSizeHint(
                QtCore.QSize(
                    widget.sizeHint().width(), widget.sizeHint().height() + 5
                )
            )
            if hasattr(widget, 'enable_version_select'):
                widget.enable_version_select = False
            self.setItemWidget(list_item, widget)
            self.addItem(list_item)
            # Find the latest published asset version.
            if not self._latest_published_asset_item:
                self._latest_published_asset_item = list_item
            else:
                if (
                    widget.version['date']
                    > self.itemWidget(
                        self._latest_published_asset_item
                    ).version['date']
                ):
                    self._latest_published_asset_item = list_item
        self.assets_added.emit(assets)
        self.currentItemChanged.connect(self.on_item_changed_callback)
        # Pre select the latest published asset version.
        self.setCurrentItem(self._latest_published_asset_item)

    def _on_version_changed_callback(self, version):
        '''Handle version changed callback'''
        self.version_changed.emit(version)

    def on_item_changed_callback(self, current_item, previous_item):
        '''Handle item changed event'''
        if hasattr(self.itemWidget(current_item), 'enable_version_select'):
            if previous_item:
                self.itemWidget(previous_item).enable_version_select = False
            self.itemWidget(current_item).enable_version_select = True
        self.selected_item_changed.emit(
            self.indexFromItem(current_item),
            self.itemWidget(current_item).version,
            self.itemWidget(current_item).asset_id,
        )

    def mousePressEvent(self, event):
        '''Override mouse press to emit signal.'''
        super(AssetList, self).mousePressEvent(event)
        self.active = True



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

from ftrack_qt.widgets.overlay import OverlayWidget
from ftrack_qt.widgets.lines import LineWidget


class OptionsButton(QtWidgets.QPushButton):
    '''Options button on publisher accordion'''

    show_overlay_signal = QtCore.Signal(object)
    hide_overlay_signal = QtCore.Signal()

    def __init__(self, title, icon, parent=None):
        '''
        Initialize options button

        :param title: The name of the step (component)
        :param icon: The button icon to use
        :param parent: the parent dialog or frame
        '''
        super(OptionsButton, self).__init__(parent=parent)
        self._title = title
        self._icon = icon
        self.__section_registry = {}

        self._main_widget = None
        self._options_widget = None
        self._overlay_container = None

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.setMinimumSize(30, 30)
        self.setMaximumSize(30, 30)
        self.setIcon(self._icon)
        self.setFlat(True)

    def build(self):
        self._main_widget = QtWidgets.QFrame()
        self._main_widget.setLayout(QtWidgets.QVBoxLayout())
        self._main_widget.layout().setAlignment(QtCore.Qt.AlignTop)

        title_label = QtWidgets.QLabel(self._title)
        title_label.setProperty('h2', True)
        self._main_widget.layout().addWidget(title_label)
        self._main_widget.layout().addWidget(QtWidgets.QLabel(''))
        self._main_widget.layout().setContentsMargins(0, 0, 0, 0)

        self._options_widget = QtWidgets.QWidget()
        self._options_widget.setLayout(QtWidgets.QVBoxLayout())

        scroll = QtWidgets.QScrollArea()
        scroll.setWidget(self._options_widget)
        scroll.setWidgetResizable(True)
        scroll.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        scroll.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)

        self._main_widget.layout().addWidget(scroll)
        # Add a vertical spacer at the end of the layout to push all widgets to the top
        vertical_spacer = QtWidgets.QSpacerItem(
            20,
            40,
            QtWidgets.QSizePolicy.Minimum,
            QtWidgets.QSizePolicy.Expanding,
        )
        self._main_widget.layout().addSpacerItem(vertical_spacer)

        self._overlay_widget = OverlayWidget()

        self._overlay_widget.set_widget(self._main_widget)

    def post_build(self):
        self.clicked.connect(self.on_click_callback)
        self._overlay_widget.close_button_clicked.connect(
            self._on_overlay_close_callback
        )

    def on_click_callback(self):
        '''Callback on clicking the options button, show the publish options overlay'''
        self.show_overlay_signal.emit(self._overlay_widget)

    def _on_overlay_close_callback(self):
        '''Emits a hide_overlay_signal when overlay close button is clicked'''
        self.hide_overlay_signal.emit()

    def add_widget(self, widget, section_name):
        if section_name not in self.__section_registry:
            self._options_widget.layout().addWidget(LineWidget())
            section_label = QtWidgets.QLabel("{}:".format(section_name))
            section_label.setProperty('secondary', True)
            self._options_widget.layout().addWidget(section_label)
            section_widget = QtWidgets.QWidget()
            section_widget_layout = QtWidgets.QVBoxLayout()
            section_widget.setLayout(section_widget_layout)
            self._options_widget.layout().addWidget(section_widget)

            self.__section_registry[section_name] = section_widget

        self.__section_registry[section_name].layout().addWidget(LineWidget())
        self.__section_registry[section_name].layout().addWidget(widget)

    def teardown(self):
        '''Delete the overlay widget and main widget'''
        self._main_widget.deleteLater()
        self._overlay_widget.deleteLater()



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore, QtGui
except ImportError:
    from PySide2 import QtWidgets, QtCore, QtGui


class StatusSelector(QtWidgets.QComboBox):
    '''A custom QComboBox for selecting statuses.'''

    statuses_changed = QtCore.Signal(object)

    @property
    def status(self):
        '''Return the currently selected status.'''
        index = self.currentIndex()
        if index > -1:
            return self.itemData(index)
        else:
            return None

    def __init__(self, parent=None):
        '''Initialize the StatusSelector.'''
        super(StatusSelector, self).__init__(parent=parent)

        self.setEditable(False)

        self.setMinimumWidth(150)
        self.setMinimumHeight(22)
        self.setMaximumHeight(22)

    def set_statuses(self, statuses):
        '''Set the available statuses in the combo box.'''
        self.clear()
        for index, status in enumerate(statuses):
            self.addItem(status['name'].upper(), status)
            # TODO: color not working.
            color = QtGui.QColor(status['color'])
            self.setItemData(
                index,
                color,
                QtCore.Qt.ItemDataRole.ForegroundRole,
            )
            self.setItemData(
                index,
                QtGui.QColor('#131920'),
                QtCore.Qt.ItemDataRole.BackgroundRole,
            )
        self.setCurrentIndex(0)

    def set_status_by_name(self, name):
        '''Set the selected status by name.'''
        for index in range(self.count()):
            if self.itemData(index)['name'] == name:
                self.setCurrentIndex(index)



from ftrack_qt.widgets.dialogs.styled_dialog import StyledDialog
from ftrack_qt.widgets.dialogs.modal_dialog import ModalDialog
from ftrack_qt.widgets.dialogs.file_dialog import FileDialog
from ftrack_qt.widgets.dialogs.scroll_tool_configs_dialog import (
    ScrollToolConfigsDialog,
)
from ftrack_qt.widgets.dialogs.tab_dialog import TabDialog
from ftrack_qt.widgets.dialogs.radio_button_dialog import RadioButtonDialog



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

from ftrack_qt.widgets.icons.status_icon import StatusMaterialIconWidget


class ArrowMaterialIconWidget(StatusMaterialIconWidget):
    '''Custom material icon widget for arrow, emitting event on click'''

    clicked = QtCore.Signal(object)

    def __init__(self, name, color=None, parent=None):
        super(ArrowMaterialIconWidget, self).__init__(
            name, color=color, parent=parent
        )

    def mousePressEvent(self, event):
        self.clicked.emit(event)
        return super(ArrowMaterialIconWidget, self).mousePressEvent(event)



from ftrack_qt.widgets.buttons.circular_button import CircularButton
from ftrack_qt.widgets.buttons.options_button import OptionsButton
from ftrack_qt.widgets.buttons.progress_button import (
    ProgressStatusButtonWidget,
    ProgressPhaseButtonWidget,
)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtCore, QtWidgets
except ImportError:
    from PySide2 import QtCore, QtWidgets

from ftrack_qt.widgets.thumbnails.base_thumbnail import ThumbnailBase


class AssetVersionThumbnail(ThumbnailBase):
    '''Asset version thumbnail widget'''

    def _download(self, url):
        '''Return thumbnail from *reference*.'''
        url = url or self.placholderThumbnail
        return super(AssetVersionThumbnail, self)._download(url)

    def _scaleAndSetPixmap(self, pixmap):
        '''Scale and set *pixmap*.'''
        if self._scale:
            scaled_pixmap = pixmap.scaled(
                self.size(),
                QtCore.Qt.AspectRatioMode.KeepAspectRatio,
                QtCore.Qt.TransformationMode.SmoothTransformation,
            )
        else:
            scaled_pixmap = pixmap
        self.setPixmap(scaled_pixmap)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

from ftrack_qt.widgets.icons import (
    ArrowMaterialIconWidget,
    MaterialIcon,
    StatusMaterialIconWidget,
)
from ftrack_qt.widgets.lines import LineWidget
from ftrack_qt.widgets.buttons import OptionsButton


class AccordionHeaderWidget(QtWidgets.QFrame):
    '''Container for accordion header - holding checkbox, title, user content
    and expander button.'''

    clicked = QtCore.Signal(object)  # User header click
    arrow_clicked = QtCore.Signal(object)  # User header click
    checkbox_status_changed = QtCore.Signal(object)
    show_options_overlay = QtCore.Signal(object)
    hide_options_overlay = QtCore.Signal()

    @property
    def title(self):
        return self._title

    @property
    def checkable(self):
        return self._checkable

    @property
    def checked(self):
        return self._checked

    @property
    def show_checkbox(self):
        return self._show_checkbox

    @property
    def collapsable(self):
        return self._collapsable

    @property
    def collapsed(self):
        return self._collapsed

    @property
    def options_button(self):
        return self._options_button

    def __init__(
        self,
        title=None,
        checkable=False,
        checked=True,
        show_checkbox=False,
        collapsable=True,
        collapsed=True,
        parent=None,
    ):
        '''
        Initialize header widget

        :param title: The default title to give header
        :param parent: The parent dialog or frame
        '''
        super(AccordionHeaderWidget, self).__init__(parent=parent)

        self._title = title
        self._checkable = checkable
        self._checked = checked
        self._show_checkbox = show_checkbox
        self._collapsable = collapsable
        self._collapsed = collapsed

        self._checkbox = None
        self._title_label = None
        self._header_content_widget = None
        self._arrow = None
        self._status = None
        self._options_button = None
        self._status_icon = None

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        self.setMinimumHeight(24)
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(2, 0, 5, 0)
        self.layout().setSpacing(10)

    def build(self):
        # Create checkbox
        self._checkbox = QtWidgets.QCheckBox()
        self._checkbox.setEnabled(self.checkable)
        self._checkbox.setChecked(self.checked)
        self._checkbox.setVisible(self.show_checkbox)

        # Create title
        self._title_label = QtWidgets.QLabel(self.title or '')
        if not self.title:
            self._title_label.hide()

        # Create Content
        self._header_content_widget = QtWidgets.QWidget()
        content_layout = QtWidgets.QHBoxLayout()
        content_layout.setContentsMargins(0, 0, 0, 0)
        content_layout.setSpacing(0)
        self._header_content_widget.setLayout(content_layout)

        # Add Status label widget in context Widget
        self._status_label = QtWidgets.QLabel()
        content_layout.addWidget(self._status_label)
        content_layout.addStretch()
        content_layout.addWidget(LineWidget(horizontal=True))
        # add options widget
        self._options_button = OptionsButton(
            self.title, MaterialIcon('settings', color='gray')
        )
        self._options_button.setProperty('borderless', True)
        content_layout.addWidget(LineWidget(horizontal=True))
        # add status icon
        self._status_icon = StatusMaterialIconWidget('check')

        # Create Arrow
        self._arrow = ArrowMaterialIconWidget(None)
        self.update_arrow_icon(self.collapsed)
        self._arrow.setVisible(self.collapsable)

        self.layout().addWidget(self._checkbox)
        self.layout().addWidget(self._title_label)
        self.layout().addWidget(self._header_content_widget, 10)
        self.layout().addWidget(self._options_button)
        self.layout().addWidget(self._status_icon)
        self.layout().addWidget(self._arrow)

    def post_build(self):
        self._checkbox.stateChanged.connect(self._on_checkbox_status_changed)
        self._arrow.clicked.connect(self._on_arrow_clicked)
        self._options_button.show_overlay_signal.connect(
            self.on_show_options_callback
        )
        self._options_button.hide_overlay_signal.connect(
            self.on_hide_options_callback
        )

    def on_show_options_callback(self, widget):
        self.show_options_overlay.emit(widget)

    def on_hide_options_callback(self):
        self.hide_options_overlay.emit()

    def add_option_widget(self, widget, section_name):
        self._options_button.add_widget(widget, section_name)

    def _on_checkbox_status_changed(self):
        self._checked = self._checkbox.isChecked()
        self._title_label.setEnabled(self._checked)
        self._header_content_widget.setEnabled(self._checked)
        self.checkbox_status_changed.emit(self._checked)

    def _on_arrow_clicked(self, event):
        self.arrow_clicked.emit(event)

    def update_arrow_icon(self, collapsed):
        '''Update the arrow icon based on collapse state'''
        if collapsed:
            icon_name = 'keyboard_arrow_down'
        else:
            icon_name = 'keyboard_arrow_up'
        self._arrow.set_icon(name=icon_name)
        self._collapsed = collapsed

    def mousePressEvent(self, event):
        '''(Override)'''
        self.clicked.emit(event)
        return super(AccordionHeaderWidget, self).mousePressEvent(event)

    def set_status(self, status, message):
        '''Set status message within header, to be implemented by child'''
        pass

    def teardown(self):
        '''Teardown the options button - properly cleanup the options overlay'''
        self._options_button.teardown()
        self._options_button.deleteLater()



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack
import time

try:
    from PySide6 import QtCore, QtWidgets
except ImportError:
    from PySide2 import QtCore, QtWidgets


class InvokeEvent(QtCore.QEvent):
    '''Event.'''

    def __init__(self, fn, *args, **kwargs):
        '''Invoke *fn* in main thread.'''
        QtCore.QEvent.__init__(
            self, QtCore.QEvent.Type(QtCore.QEvent.registerEventType())
        )
        self.fn = fn
        self.args = args
        self.kwargs = kwargs
        self.result = None
        self.triggered = False


class Invoker(QtCore.QObject):
    '''Invoker.'''

    def event(self, event):
        '''Call function on *event*.'''
        event.result = event.fn(*event.args, **event.kwargs)
        event.triggered = True
        return True


_invoker = Invoker(None)


def invoke_in_qt_main_thread(fn, *args, **kwargs):
    '''
    Invoke function *fn* with arguments, if not running in the main thread.

    TODO: Align this with DCC utility functions to run in main thread, and use them
    instead as their QT implementation might differ and this solution might not apply
    or cause instabilities.
    '''
    if QtCore.QThread.currentThread() is _invoker.thread():
        return fn(*args, **kwargs)
    else:
        invoke_event = InvokeEvent(fn, *args, **kwargs)
        QtCore.QCoreApplication.postEvent(_invoker, invoke_event)
        # Wait for event to be run
        while not invoke_event.triggered:
            time.sleep(0.01)
            QtWidgets.QApplication.processEvents()
        return invoke_event.result



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack
# TODO: Clean this code
try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

from ftrack_qt.widgets.thumbnails import UserThumbnail as UserThumbnail

# Cache of user names.
NAME_CACHE = dict()


class FtrackUser(QtWidgets.QFrame):
    '''Header user avatar widget'''

    def __init__(self, session, parent=None):
        '''Instantiate user name and logo widget using *username*.'''

        super(FtrackUser, self).__init__(parent=parent)
        self.session = session

        self.pre_build()
        self.build()

    def pre_build(self):
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setAlignment(QtCore.Qt.AlignmentFlag.AlignRight)

    def build(self):
        username = self.session.api_user
        # self.label = QtWidgets.QLabel(self)
        self.image = UserThumbnail(self.session)
        self.image.setFixedSize(35, 35)

        self.layout().addWidget(self.image)

        self.image.load(username)

        if username not in NAME_CACHE:
            user = self.session.query(
                'select first_name, last_name from User where username is "{}"'.format(
                    username
                )
            ).first()

            NAME_CACHE[username] = '{} {}'.format(
                user['first_name'], user['last_name']
            ).title()

        tooltip = 'Logged in as: {}'.format(self.session.api_user)
        tooltip += '\n'
        tooltip += 'Server: {}'.format(self.session.server_url)
        tooltip += '\n'
        location = self.session.pick_location()
        if location:
            tooltip += 'Location: {}'.format(location['name'])
            tooltip += '\n'
            tooltip += 'Priority: {}'.format(location.priority)
            tooltip += '\n'
            if location.accessor:
                tooltip += 'Accessor: {}'.format(location.accessor.__module__)
                if hasattr(location.accessor, 'prefix'):
                    tooltip += ' @ {}'.format(location.accessor.prefix)
                tooltip += '\n'
            if location.structure:
                tooltip += 'Structure: {}\n'.format(
                    location.structure.__module__
                )
        else:
            tooltip += 'Location: - not set -'
        tooltip += '\n'
        self.setToolTip(tooltip)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack
import logging

try:
    from PySide6 import QtWidgets, QtCore
    import shiboken6 as shiboken
except ImportError:
    from PySide2 import QtWidgets, QtCore
    import shiboken2 as shiboken

from ftrack_qt.widgets.info import EntityInfo

from ftrack_qt.widgets.thumbnails import ContextThumbnail
from ftrack_utils.threading import BaseThread
from ftrack_qt.widgets.buttons import CircularButton

# TODO: this code has to be reviewed


class ContextSelector(QtWidgets.QFrame):
    '''Widget representing the current context by showing entity info, and a button enabling change of context'''

    change_context_clicked = QtCore.Signal()
    entity_changed = QtCore.Signal(object)
    context_changed = QtCore.Signal(object)
    entity_found = QtCore.Signal(object)
    disable_thumbnail = False

    @property
    def entity(self):
        '''Return the entity'''
        return self._entity

    @entity.setter
    def entity(self, value):
        '''Set the entity to *value*'''
        # prevent setting the same entity twice
        if value == self._entity:
            return
        self._entity = value
        self.entity_info.entity = value
        self._context_id = value['id'] if value else None
        if not self.disable_thumbnail:
            self.set_thumbnail(self._entity)
        self.entity_changed.emit(value)
        self.context_changed.emit(value['id'])
        self.entity_info.setVisible(self.entity is not None)
        self.no_entity_label.setVisible(self.entity is None)

    @property
    def context_id(self):
        '''Return the context id'''
        return self._context_id

    @context_id.setter
    def context_id(self, value):
        '''Set the entity context id to *value*'''
        if not value:
            return
        thread = BaseThread(
            name='find-context-entity-thread',
            target=self._find_context_entity,
            callback=self._on_entity_found_async,
            target_args=[value],
        )
        self.__thread_registry.append(thread)
        self._browsing_context = True
        thread.start()

    @property
    def is_browsing(self):
        browsing = False
        for thread in self.__thread_registry:
            if thread.is_alive() or self._browsing_context:
                browsing = True
                break
        return browsing

    @property
    def browse_context_id(self):
        '''Return the browse context id'''
        return self._browse_context_id

    @browse_context_id.setter
    def browse_context_id(self, value):
        '''Set the initial browse context id'''
        self._browse_context_id = value

    @property
    def entity_browser(self):
        '''Return the entity browser'''
        return self._entity_browser

    def __init__(
        self,
        session,
        enable_context_change=False,
        select_task=True,
        browse_context_id=None,
        parent=None,
    ):
        '''
        Initialise ContextSelector widget

        :param session: :class:`ftrack_api.session.Session`
        :param enable_context_change:  If set to True, this contest selection is allowed to spawn the entity browser
            and change global context.
        :param select_task: If true. only tasks can be selected in the entity browser. If false, any context can be selected.
        :param browse_context_id: If set, the entity browser will be opened with this context id as the root.
        :param parent: The parent dialog or frame
        '''

        super(ContextSelector, self).__init__(parent=parent)

        self.logger = logging.getLogger(__name__)

        self._enable_context_change = enable_context_change
        self._select_task = select_task
        self._browse_context_id = browse_context_id
        self._entity = None
        self._context_id = None
        self.session = session
        self.__thread_registry = []
        self._browsing_context = False
        self._entity_browser = None

        self.pre_build()
        self.build()
        self.post_build()

    def pre_build(self):
        layout = QtWidgets.QHBoxLayout()
        layout.setContentsMargins(10, 1, 10, 1)
        layout.setSpacing(8)
        self.setLayout(layout)

    def build(self):
        from ftrack_qt.widgets.browsers import (
            EntityBrowser,
        )  # Prevent circular import

        self.thumbnail_widget = ContextThumbnail(self.session)

        self.thumbnail_widget.setMinimumWidth(50)
        self.thumbnail_widget.setMinimumHeight(40)
        self.thumbnail_widget.setMaximumWidth(50)
        self.thumbnail_widget.setMaximumHeight(40)

        self.entity_info = EntityInfo()
        self.entity_info.setMinimumHeight(40)
        self.entity_info.setMaximumHeight(40)
        self.entity_info.setVisible(False)

        self.no_entity_label = QtWidgets.QLabel(
            '<html><i>Please select an entity!</i></html>'
        )
        self.no_entity_label.setVisible(True)

        self.entity_browse_button = CircularButton('edit', variant='outlined')

        self.layout().addWidget(self.thumbnail_widget)
        self.layout().addWidget(self.entity_info)
        self.layout().addWidget(self.no_entity_label)
        self.layout().addWidget(QtWidgets.QLabel(), 10)
        self.layout().addWidget(self.entity_browse_button)

        # Build entity browser:

        self._entity_browser = EntityBrowser(
            self.parent(),
            self.session,
            title='CHOOSE TASK (WORKING CONTEXT)'
            if self._select_task
            else 'CHOOSE CONTEXT',
            mode=EntityBrowser.MODE_TASK
            if self._select_task
            else EntityBrowser.MODE_CONTEXT,
        )
        self.entity_browser.setMinimumWidth(600)
        if self._entity:
            self.entity_browser.entity = self._entity['parent']
        elif self._browse_context_id:
            self.entity_browser.entity_id = self._browse_context_id
        else:
            self.entity_browser.entity = None
        if self.entity_browser.entity:
            self.entity = self.entity_browser.entity

    def set_thumbnail(self, entity):
        '''Set the entity thumbnail'''
        self.thumbnail_widget.load(entity['id'])

    def post_build(self):
        self.entity_browse_button.clicked.connect(
            self._on_entity_browse_button_clicked
        )
        self.entity_found.connect(self._on_entity_found)
        self.setMaximumHeight(50)

    def reset(self, entity=None):
        '''Reset browser to the given *entity* or the default one'''
        current_entity = entity or self._entity
        self.entity = current_entity

    def _find_context_entity(self, context_id):
        '''(Run in background thread) Query entity from ftrack'''
        context_entity = self.session.query(
            'select link, name , parent, parent.name from Context where id '
            'is "{}"'.format(context_id)
        ).one()
        return context_entity

    def _on_entity_found_async(self, entity):
        '''(Run in background thread) Entity found callback'''
        if not shiboken.isValid(self):
            # Widget has been closed while entity fetched
            self._browsing_context = False
            return
        self.entity_found.emit(entity)

    def _on_entity_found(self, entity):
        '''Entity found callback, set entity'''
        self.entity = entity
        self._browsing_context = False

    def _on_entity_browse_button_clicked(self):
        '''Handle entity browse button clicked'''

        if self._enable_context_change:
            # Launch browser.
            self.entity_browser.entity = self.entity
            if self.entity_browser.exec_():
                self.entity = self.entity_browser.entity
        else:
            # Let client decide what to do when user wants to change context
            self.change_context_clicked.emit()

    def teardown(self):
        if self._entity_browser:
            self._entity_browser.deleteLater()



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack
import logging

try:
    from PySide6 import QtWidgets, QtCore, QtGui
except ImportError:
    from PySide2 import QtWidgets, QtCore, QtGui


class VersionSelector(QtWidgets.QComboBox):
    '''Version selector combobox'''

    @property
    def version(self):
        '''Return current selected asset version entity'''
        index = self.currentIndex()
        if index > -1:
            return self.itemData(index)
        else:
            return None

    def __init__(self, parent=None):
        '''Initialize the VersionSelector.'''
        super(VersionSelector, self).__init__(parent=parent)
        self.logger = logging.getLogger(
            __name__ + '.' + self.__class__.__name__
        )

        self.setEditable(False)

        self.setMaximumHeight(24)
        self.setMinimumHeight(24)

    def set_versions(self, versions):
        '''Set the versions in the combobox.'''
        self.clear()
        self._add_versions(sorted(versions, key=lambda v: -v['version']))

    def _add_versions(self, sorted_versions):
        '''Add versions to the combobox.'''
        for index, version in enumerate(sorted_versions):
            self._add_version(version)
        self.setCurrentIndex(0)

    def _add_version(self, version):
        '''Add a version to the combobox.'''
        self.addItem(str('v{}'.format(version['version'])), version)
        self.setItemData(
            self.count() - 1,
            QtGui.QColor('#131920'),
            QtCore.Qt.ItemDataRole.BackgroundRole,
        )



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack
# TODO: Clean this code

import urllib.request, urllib.parse, urllib.error

from ftrack_qt.widgets.thumbnails.ellipse_thumbnail import EllipseThumbnailBase


class UserThumbnail(EllipseThumbnailBase):
    '''User(avatar) thumbnail widget'''

    def _download(self, reference):
        '''Return thumbnail from *reference*.'''
        thumbnail = self.session.query(
            'select thumbnail from User where username is "{}"'.format(
                reference
            )
        ).first()['thumbnail']
        url = self.get_thumbnail_url(thumbnail)
        return super(UserThumbnail, self)._download(url)

    def get_thumbnail_url(self, component):
        if not component:
            return

        params = urllib.parse.urlencode(
            {
                'id': component['id'],
                'username': self.session.api_user,
                'apiKey': self.session.api_key,
            }
        )

        result_url = '{base_url}/component/thumbnail?{params}'.format(
            base_url=self.session._server_url, params=params
        )
        return result_url



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore, QtGui
except ImportError:
    from PySide2 import QtWidgets, QtCore, QtGui


class ShadedWidget(QtWidgets.QWidget):
    '''The overlay widget used to shade the dialog'''

    def __init__(self, parent=None):
        super(ShadedWidget, self).__init__(parent=parent)
        self.setWindowFlags(QtCore.Qt.WindowType.FramelessWindowHint)
        self.setAttribute(QtCore.Qt.WidgetAttribute.WA_TranslucentBackground)
        self._fill_color = QtGui.QColor(19, 25, 32, 169)

    def paintEvent(self, event):
        super(ShadedWidget, self).paintEvent(event)
        # Get current window size and paint a semi transparent dark overlay across widget
        size = self.size()
        painter = QtGui.QPainter()
        painter.begin(self)
        painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
        painter.setPen(self._fill_color)
        painter.setBrush(self._fill_color)
        painter.drawRect(0, 0, size.width(), size.height())
        painter.end()



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore, QtGui
except ImportError:
    from PySide2 import QtWidgets, QtCore, QtGui


class FtrackLogo(QtWidgets.QLabel):
    '''Header logo widget'''

    def __init__(self, parent=None):
        '''Instantiate logo widget.'''
        super(FtrackLogo, self).__init__(parent=parent)

        self.pre_build()
        self.build()

    def pre_build(self):
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(0)
        self.layout().setAlignment(QtCore.Qt.AlignmentFlag.AlignTop)

    def build(self):
        resource_path = ':ftrack/image/default/connectLogoDark'
        logoPixmap = QtGui.QPixmap(resource_path)
        if not logoPixmap is None:
            self.setPixmap(
                logoPixmap.scaled(
                    QtCore.QSize(106, 32),
                    QtCore.Qt.AspectRatioMode.KeepAspectRatio,
                    QtCore.Qt.TransformationMode.SmoothTransformation,
                )
            )
            self.setPixmap(logoPixmap)
        else:
            self.setText("ftrack")



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

from ftrack_qt.utils.widget import set_property
from ftrack_qt.widgets.headers import AccordionHeaderWidget


class AccordionBaseWidget(QtWidgets.QFrame):
    '''A utility accordion widget providing a header which can be expanded to show content'''

    clicked = QtCore.Signal(object)  # Emitted when accordion is clicked
    doubleClicked = QtCore.Signal(
        object
    )  # Emitted when accordion is double clicked
    show_options_overlay = QtCore.Signal(object)
    hide_options_overlay = QtCore.Signal()
    enabled_changed = QtCore.Signal(object)

    @property
    def title(self):
        '''Return the title text shown in header by default'''
        return self._title

    @property
    def selectable(self):
        '''Return the current selection mode'''
        return self._selectable

    @property
    def checkable(self):
        '''Return True if accordion is checkable with a checkbox'''
        return self._checkable

    @property
    def show_checkbox(self):
        '''Return True if accordion is checkable with a checkbox'''
        return self._show_checkbox

    @property
    def collapsed(self):
        '''Return True if accordion is collapsed - content hidden (default)'''
        return self._collapsed

    @property
    def collapsable(self):
        '''Return True if accordion is collapsable - content can be hidden (default)'''
        return self._collapsable

    @property
    def selected(self):
        '''Return True if accordion is selected'''
        return self._selected

    @selected.setter
    def selected(self, value):
        '''Set accordion selected property'''
        if self.isEnabled() and self.selectable:
            self._selected = value
            self.update_accordion()

    @property
    def checked(self):
        '''(Checkable) Return True if checked'''
        return self._checked

    def __init__(
        self,
        selectable=False,
        show_checkbox=False,
        checkable=False,
        title=None,
        selected=False,
        checked=True,
        collapsable=True,
        collapsed=True,
        parent=None,
    ):
        '''
        Initialize base accordion widget

        :param select_mode: Mode telling if accordion should be selectable or not
        :param check_mode: Mode telling if accordion should be checkale or not
        :param title: The default title text to show in accordion header
        :param selected: Flag telling if accordion should be initially selected or not
        :param checked: Flag telling if accordion should be initially checked or not
        :param collapsable: Flag telling if accordion is collapsable or not
        :param collapsed: Flag telling if accordion should be initially collapsed or not
        :param docked: Flag telling if accordion is docked in DCC or within an ftrack dialog - drives the style
        :param parent:  the parent dialog or frame
        '''
        super(AccordionBaseWidget, self).__init__(parent=parent)

        self._indicator_widget = None
        self._header_widget = None
        self._content_widget = None

        self._selectable = selectable
        self._checkable = checkable
        self._show_checkbox = show_checkbox
        self._title = title
        self._collapsable = collapsable

        self._selected = selected
        self._checked = checked
        self._collapsed = collapsed

        self.pre_build()
        self.build()
        self.post_build()

    def set_status(self, status, message):
        self._header_widget.set_status(status, message)

    def pre_build(self):
        self.setLayout(QtWidgets.QHBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(0)

    def build(self):
        # Create the indicator widget
        self._indicator_widget = QtWidgets.QFrame()
        self._indicator_widget.setMaximumWidth(4)
        self._indicator_widget.setMinimumWidth(4)
        self._indicator_widget.setVisible(False)

        self.layout().addWidget(self._indicator_widget)

        # Create the main_widget
        main_widget = QtWidgets.QWidget()
        main_widget.setLayout(QtWidgets.QVBoxLayout())
        main_widget.layout().setAlignment(QtCore.Qt.AlignmentFlag.AlignTop)
        main_widget.layout().setContentsMargins(0, 0, 0, 0)
        main_widget.layout().setSpacing(1)

        # Create Header
        self._header_widget = AccordionHeaderWidget(
            title=self.title,
            checkable=self.checkable,
            checked=self.checked,
            show_checkbox=self.show_checkbox,
            collapsable=self.collapsable,
            collapsed=self.collapsed,
        )

        # Add header to main widget
        main_widget.layout().addWidget(self._header_widget)

        # Create content widget
        self._content_widget = QtWidgets.QFrame()
        self._content_widget.setLayout(QtWidgets.QVBoxLayout())
        self._content_widget.layout().setContentsMargins(2, 2, 2, 2)
        self._content_widget.layout().setSpacing(0)

        # Add content widget to main widget
        main_widget.layout().addWidget(self._content_widget)

        # Add main_widget to main layout
        self.layout().addWidget(main_widget)

    def post_build(self):
        self._header_widget.checkbox_status_changed.connect(
            self._on_checkbox_status_changed
        )
        self._header_widget.clicked.connect(self._on_header_clicked)
        self._header_widget.arrow_clicked.connect(
            self._on_header_arrow_clicked
        )
        self._header_widget.show_options_overlay.connect(
            self._on_show_options_overlay_callback
        )
        self._header_widget.hide_options_overlay.connect(
            self._on_hide_options_overlay_callback
        )
        self._content_widget.setVisible(not self._collapsed)
        self._content_widget.setEnabled(self.checked)

    def _on_show_options_overlay_callback(self, widget):
        self.show_options_overlay.emit(widget)

    def _on_hide_options_overlay_callback(self):
        self.hide_options_overlay.emit()

    def add_option_widget(self, widget, section_name):
        self._header_widget.add_option_widget(widget, section_name)

    def add_widget(self, widget):
        '''Add widget to content'''
        self._content_widget.layout().addWidget(widget)

    def count_widgets(self):
        '''Return the number of widgets within the content'''
        return self._content_widget.layout().count()

    def get_widget_at(self, index):
        '''Return the content widget at *index*'''
        return self._content_widget.layout().itemAt(index).widget()

    def paint_title(self, color):
        '''Put a foreground *color* on header title label'''
        self._header_widget._title_label.setStyleSheet(
            "color: {}".format(color)
        )

    def set_indicator_color(self, color):
        '''Set the left indicator visibility depending on *color*'''
        set_property(
            self._indicator_widget,
            'indicator',
            color or 'green',
        )
        if not self._indicator_widget.isVisible():
            self._indicator_widget.setVisible(True)

    def _on_checkbox_status_changed(self, checked):
        '''Callback on enable checkbox user interaction'''
        self._checked = checked
        self._content_widget.setEnabled(self.checked)
        self.enabled_changed.emit(self.checked)

    def _on_header_clicked(self, event):
        '''Callback on header user click'''
        if not self.selectable:
            if event.button() != (QtCore.Qt.MouseButton.RightButton):
                self.toggle_collapsed()

    def _on_header_arrow_clicked(self, event):
        '''Callback on header arrow user click'''
        if self.selectable:
            # This is the way to collapse
            self.toggle_collapsed()

    def toggle_collapsed(self):
        '''Toggle the accordion collapsed state'''
        self._collapsed = not self._collapsed
        self._header_widget.update_arrow_icon(int(self.collapsed))
        self._on_collapse(self.collapsed)
        self._content_widget.setVisible(not self._collapsed)

    def _on_collapse(self, collapsed):
        '''(Optional) To be overridden by child'''
        pass

    def mousePressEvent(self, event):
        '''(Override)'''
        self.clicked.emit(event)
        self.on_click(event)
        return super(AccordionBaseWidget, self).mousePressEvent(event)

    def on_click(self, event):
        '''Callback on accordion user click, to be overridden by child'''
        pass

    def mouseDoubleClickEvent(self, event):
        '''(Override)'''
        self.doubleClicked.emit(event)
        self.on_double_click(event)
        return super(AccordionBaseWidget, self).mouseDoubleClickEvent(event)

    def on_double_click(self, event):
        '''Callback on accordion user double click, to be overridden by child'''
        self.toggle_collapsed()

    def update_accordion(self):
        '''Update accordion background depending on selection state'''
        if self.selectable:
            set_property(
                self,
                'background',
                'selected' if self._selected else 'transparent',
            )

    def teardown(self):
        '''Teardown the header widget - properly cleanup the options overlay'''
        self._header_widget.teardown()
        self._header_widget.deleteLater()



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack

try:
    from PySide6 import QtWidgets, QtCore
except ImportError:
    from PySide2 import QtWidgets, QtCore

# TODO: try to generalize this delegate to be able to pass the looking keys.
#  So it can be used in multiple places and not only for asset versions.


class AssetVersionComboBoxDelegate(QtWidgets.QStyledItemDelegate):
    '''
    QItemDelegate that provides a combo box to select versions of an
    asset_version
    '''

    index_changed = QtCore.Signal(object, object)

    def __init__(self, parent=None):
        super(AssetVersionComboBoxDelegate, self).__init__(parent=parent)

    def createEditor(self, parent, option, index):
        '''Creates the ComboBox'''
        item = index.model().data(index, index.model().DATA_ROLE)

        versions_collection = item['asset']['versions']
        combo = QtWidgets.QComboBox(parent)
        for asset_version in versions_collection:
            combo.addItem(str(asset_version['version']), asset_version)

        return combo

    def setEditorData(self, editor, index):
        '''Sets the given *data* into the given *editor*'''
        editor_data = str(
            index.model().data(index, QtCore.Qt.ItemDataRole.EditRole)
        )
        idx = editor.findText(editor_data)
        editor.setCurrentIndex(idx)

    def setModelData(self, editor, model, index):
        '''
        Emits the signal index_changed when a new version is been selected
        '''
        index.data()
        if not index.isValid():
            return False
        data = editor.itemData(editor.currentIndex())
        if not data:
            return False
        self.index_changed.emit(index, data)



# :coding: utf-8
# :copyright: Copyright (c) 2024 ftrack


try:
    from PySide6 import QtWidgets, QtCore, QtGui
    import shiboken6 as shiboken
except ImportError:
    from PySide2 import QtWidgets, QtCore, QtGui
    import shiboken2 as shiboken

from ftrack_utils.framework.config.tool import get_plugins


def check_framework_dialog_bases(cls):
    '''Recursively check the base classes for FrameworkDialog.'''
    for base in cls.__bases__:
        if base.__name__ == 'FrameworkDialog':
            return base
        if check_framework_dialog_bases(base):
            return base
    return False


def get_main_window_from_widget(widget):
    '''Return the main window from the given widget.'''
    return widget.window()


def get_framework_main_dialog(widget):
    '''Return the main Framework dialog from the given widget.'''
    main_dialog = None
    while not main_dialog:
        widget = widget.parentWidget()
        if not widget:
            break
        if check_framework_dialog_bases(widget.__class__):
            main_dialog = widget

    return main_dialog


def set_property(widget, name, value):
    '''Update property *name* to *value* for *widget*, and polish afterwards.'''
    widget.setProperty(name, value)
    if widget.style() is not None and shiboken.isValid(
        widget.style()
    ):  # Only update style if applied and valid
        widget.style().unpolish(widget)
        widget.style().polish(widget)
    widget.update()


def set_properties(widget, properties):
    """
    Set multiple properties on a Qt widget.

    Args:
        widget (QWidget): The widget on which to set the properties.
        properties (dict): A dictionary of property names and values.
    """
    for name, value in properties.items():
        widget.setProperty(name, value)
        if widget.style() is not None and shiboken.isValid(
            widget.style()
        ):  # Only update style if applied and valid
            widget.style().unpolish(widget)
            widget.style().polish(widget)
        widget.update()


def center_widget(widget, width=None, height=None):
    '''Returns a widget that is *widget* centered horizontally and vertically'''
    v_container = QtWidgets.QWidget()
    v_container.setLayout(QtWidgets.QVBoxLayout())
    v_container.layout().addWidget(QtWidgets.QLabel(""), 100)

    h_container = QtWidgets.QWidget()
    h_container.setLayout(QtWidgets.QHBoxLayout())
    h_container.layout().addWidget(QtWidgets.QLabel(), 100)
    h_container.layout().addWidget(widget)
    if not width is None and not height is None:
        widget.setMaximumSize(QtCore.QSize(width, height))
        widget.setMinimumSize(QtCore.QSize(width, height))
    h_container.layout().addWidget(QtWidgets.QLabel(), 100)
    v_container.layout().addWidget(h_container)

    v_container.layout().addWidget(QtWidgets.QLabel(), 100)
    return v_container


class InputEventBlockingWidget(QtWidgets.QWidget):
    '''Conditional input event blocking widget'''

    def __init__(self, blocker, parent=None):
        '''
        Initialize InputEventBlockingWidget

        :param blocker: Blocker function that should return true if event should be blocked
        :param parent: The parent dialog or frame
        '''
        super(InputEventBlockingWidget, self).__init__(parent=parent)
        self._blocker = blocker
        self._filtered_widget = QtCore.QCoreApplication.instance()
        self._filtered_widget.installEventFilter(self)

    def stop(self):
        self._filtered_widget.removeEventFilter(self)

    def eventFilter(self, obj, event):
        '''Block *event* on *obj* while if blocker returns True'''
        retval = False
        if self._blocker() and (isinstance(event, QtGui.QInputEvent)):
            retval = True
        return retval


def build_progress_data(tool_config):
    '''Build progress data from *tool_config*'''
    progress_data = []
    for plugin_config in get_plugins(tool_config, with_parents=True):
        enabled = True
        if plugin_config.get('enabled') == False:
            enabled = False
            continue
        phase_data = {
            'id': plugin_config['reference'],
            'label': plugin_config['plugin'].replace('_', ' ').title(),
        }
        tags = plugin_config.get('tags') or []
        for group in plugin_config.get('parents') or []:
            if group.get('enabled') == False:
                enabled = False
                continue
            if 'options' in group:
                tags.extend(list(str(group['options'].values())))
            if 'tags' in group:
                tags.extend(group['tags'])
        if enabled:
            phase_data['tags'] = reversed(tags)
            progress_data.append(phase_data)
    return progress_data
